﻿Imports MVVM.Client.Data.DataServices
Imports System.Data.Services.Client
Imports System.Linq

Public Class GameCatalog
  Implements IGameCatalog

  Private theServiceRoot As Uri
  Private theEntities As GamesEntities
  Private Const MAX_RESULTS As Integer = 50

  Public Sub New()
    Me.New(New Uri("/Games.svc", UriKind.Relative))
  End Sub

  Public Sub New(ByVal serviceRoot As Uri)
    theServiceRoot = serviceRoot
  End Sub

  Public Event GameLoadingComplete As EventHandler(Of GameLoadingEventArgs) _
    Implements IGameCatalog.GameLoadingComplete
  Public Event GameLoadingError As EventHandler(Of GameCatalogErrorEventArgs) _
    Implements IGameCatalog.GameLoadingError
  Public Event GameSavingComplete As EventHandler _
    Implements IGameCatalog.GameSavingComplete
  Public Event GameSavingError As EventHandler(Of GameCatalogErrorEventArgs) _
    Implements IGameCatalog.GameSavingError

  Public Sub GetGames() Implements IGameCatalog.GetGames
    ' Get all the games ordered by release date
    Dim qry = From g In Entities.Games _
              Order By g.ReleaseDate Descending _
              Take MAX_RESULTS _
              Select g

    ExecuteGameQuery(qry)
  End Sub

  Public Sub GetGamesByGenre(ByVal genre As String) _
    Implements IGameCatalog.GetGamesByGenre

    ' Get all the games ordered by release date
    Dim qry = From g In Entities.Games _
              Where g.Genre.ToLower() = genre.ToLower() _
              Order By g.ReleaseDate _
              Select g

    ExecuteGameQuery(qry)
  End Sub

  Public Sub SaveChanges() Implements IGameCatalog.SaveChanges
    ' Save Not Yet Implemented
    Throw New NotImplementedException()
  End Sub

  ' Call the query asynchronously and add the results to the collection
  Private Sub ExecuteGameQuery(ByVal qry As DataServiceQuery(Of Game))
    ' Execute the query
    qry.BeginExecute(New AsyncCallback(AddressOf GameQueryComplete), qry)
  End Sub

  Sub GameQueryComplete(ByVal result As IAsyncResult)

    Dim qry As DataServiceQuery(Of Game) = _
      CType(result.AsyncState, DataServiceQuery(Of Game))

    Try
      Dim results As IEnumerable(Of Game) = qry.EndExecute(result)

      RaiseEvent GameLoadingComplete(Me, New GameLoadingEventArgs(results))
    Catch ex As Exception
      RaiseEvent GameLoadingError(Me, New GameCatalogErrorEventArgs(ex))
    End Try

  End Sub

  Private ReadOnly Property Entities() As GamesEntities
    Get
      If theEntities Is Nothing Then
        theEntities = New GamesEntities(theServiceRoot)
      End If
      Return theEntities
    End Get
  End Property

End Class