﻿Imports MVVM.Client.Data.DataServices
Imports System.Collections.ObjectModel

Public Class GamesViewModel
  Private theCatalog As IGameCatalog
  Private theGames As ObservableCollection(Of Game) = New ObservableCollection(Of Game)()

  Public Event LoadComplete As EventHandler
  Public Event ErrorLoading As EventHandler

  Public Sub New()
    Me.New(New GameCatalog())
  End Sub

  Public Sub New(ByVal catalog As IGameCatalog)
    theCatalog = catalog
    AddHandler theCatalog.GameLoadingComplete, AddressOf games_GameLoadingComplete
    AddHandler theCatalog.GameLoadingError, AddressOf games_GameLoadingError
  End Sub

  Private Sub games_GameLoadingError(ByVal sender As Object, _
                                     ByVal e As GameCatalogErrorEventArgs)
    ' Fire Event on UI Thread
    Application.Current.RootVisual.Dispatcher.BeginInvoke(AddressOf RaiseErrorLoading)

  End Sub

  Sub RaiseErrorLoading()
    RaiseEvent ErrorLoading(Me, Nothing)
  End Sub

  Private Sub games_GameLoadingComplete(ByVal sender As Object, _
                                        ByVal e As GameLoadingEventArgs)

    ' Clear the list
    theGames.Clear()

    ' Add the new games
    For Each g As Game In e.Results
      theGames.Add(g)
    Next g

    ' Fire Event on UI Thread
    Application.Current.RootVisual.Dispatcher.BeginInvoke(AddressOf RaiseLoadComplete)

  End Sub

  Sub RaiseLoadComplete()
    RaiseEvent LoadComplete(Me, Nothing)
  End Sub

  Public Sub LoadGames()
    theCatalog.GetGames()
  End Sub

  Public Sub LoadGamesByGenre(ByVal genre As String)
    theCatalog.GetGamesByGenre(genre)
  End Sub

  Public ReadOnly Property Games() As ObservableCollection(Of Game)
    Get
      Return theGames
    End Get
  End Property
End Class