﻿Imports Microsoft.Silverlight.Testing.UnitTesting.Metadata.VisualStudio
Imports Microsoft.Silverlight.Testing
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports MVVM.Client.Data
Imports MVVM.Client.Test.Mocks

' Implementation is more complex because VB does not
' Support Anonymous Delegates or non-Function Lambdas

Namespace MVVM.Client.Test

  <TestClass()> _
  Public Class GameModelTests
    Inherits SilverlightTest

    Dim WithEvents viewModel As GamesViewModel
    Dim loadComplete As Boolean = False

    Sub viewModel_Loaded() Handles viewModel.LoadComplete
      loadComplete = True
    End Sub

    <TestInitialize()> _
    Public Sub TestInit()

      viewModel = New GamesViewModel(New MockGameCatalog())

    End Sub

    <TestMethod(), _
     Description("Test LoadGames from the ViewModel"), _
     Asynchronous()> _
    Public Sub TestLoadGames()

      loadComplete = False
      viewModel.LoadGames()

      EnqueueConditional(Function() loadComplete)
      EnqueueCallback(AddressOf TestLoadGamesEnqueuedTests)
      EnqueueTestComplete()

    End Sub

    Sub TestLoadGamesEnqueuedTests()

      Assert.AreNotEqual(viewModel.Games, Nothing, "Expected games list not to be null.")
      Assert.IsTrue(viewModel.Games.Count > 0, "Expected games list have results.")

    End Sub

    <TestMethod(), _
     Description("Test Successful LoadGamesByGenre from the ViewModel"), _
     Asynchronous()> _
    Public Sub TestLoadGamesByGenre()

      loadComplete = False

      viewModel.LoadGamesByGenre("Family")

      EnqueueConditional(Function() loadComplete)
      EnqueueCallback(AddressOf TestLoadGamesByGenreEnqueuedTests)
      EnqueueTestComplete()

    End Sub

    Sub TestLoadGamesByGenreEnqueuedTests()

      Assert.AreNotEqual(viewModel.Games, Nothing, "Expected games list not to be null.")
      Assert.IsTrue(viewModel.Games.Count > 0, "Expected games list have results.")

    End Sub


    <TestMethod(), Description("Test Failed LoadGamesByGenre from the ViewModel"), Asynchronous()> _
    Public Sub TestLoadGamesByGenreWithoutResults()

      loadComplete = False

      viewModel.LoadGamesByGenre("Mature") ' Non-existent Genre

      EnqueueConditional(Function() loadComplete)
      EnqueueCallback(AddressOf TestLoadGamesByGenreWithoutResultsTests)
      EnqueueTestComplete()

    End Sub

    Sub TestLoadGamesByGenreWithoutResultsTests()
      Assert.AreNotEqual(viewModel.Games, Nothing, "Expected games list not to be null.")
      Assert.IsTrue(viewModel.Games.Count = 0, "Expected games list have no results.")
    End Sub
  End Class
End Namespace