﻿Imports MVVM.Client.Data
Imports System.Windows.Threading
Imports MVVM.Client.Data.DataServices

' Mock Implementation is more complex because VB does not
' Support Anonymous Delegates or non-Function Lambdas

Namespace Mocks

  Public Class MockGameCatalog
    Implements IGameCatalog

    Private Const DELAY As Integer = 1000
    Private currentGenre As String

    Public Sub GetGames() Implements IGameCatalog.GetGames

      Dim timer As New DispatcherTimer() With _
      { _
        .Interval = TimeSpan.FromMilliseconds(DELAY) _
      }

      AddHandler timer.Tick, AddressOf OnGamesTimerTick

      timer.Start()

    End Sub

    Public Sub GetGamesByGenre(ByVal genre As String) _
      Implements IGameCatalog.GetGamesByGenre

      currentGenre = genre

      Dim timer As New DispatcherTimer() With _
      { _
        .Interval = TimeSpan.FromMilliseconds(DELAY) _
      }

      AddHandler timer.Tick, AddressOf OnGenreTimerTick

      timer.Start()
    End Sub

    Public Sub SaveChanges() Implements IGameCatalog.SaveChanges
      Throw New NotImplementedException()
    End Sub

    Private Function MockData() As IEnumerable(Of Game)
      ' Create a sample game list
      Return New Game() {New Game() With {.ProductName = "Some Game", .Genre = "Family", .Rating = "E (Everyone)", .ReleaseDate = DateTime.Parse("04/04/2008")}, New Game() With {.ProductName = "Some Violent Game", .Genre = "Shooter", .Rating = "M (Mature)", .ReleaseDate = DateTime.Parse("04/04/2008")}}
    End Function

    Private Sub OnGamesTimerTick(ByVal sender As Object, ByVal e As EventArgs)

      If GameLoadingCompleteEvent IsNot Nothing Then
        Dim filtered = From g In MockData() _
                       Select g

        RaiseEvent GameLoadingComplete(Me, New GameLoadingEventArgs(filtered.AsEnumerable))
      End If

    End Sub

    Private Sub OnGenreTimerTick(ByVal sender As Object, ByVal e As EventArgs)

      If GameLoadingCompleteEvent IsNot Nothing Then
        Dim filtered = From g In MockData() _
                       Where g.Genre = currentGenre _
                       Select g

        RaiseEvent GameLoadingComplete(Me, New GameLoadingEventArgs(filtered.AsEnumerable))
      End If

    End Sub

    Public Event GameLoadingComplete As  _
      EventHandler(Of GameLoadingEventArgs) _
      Implements IGameCatalog.GameLoadingComplete
    Public Event GameLoadingError As  _
      EventHandler(Of GameCatalogErrorEventArgs) _
      Implements IGameCatalog.GameLoadingError
    Public Event GameSavingComplete _
      As EventHandler _
      Implements IGameCatalog.GameSavingComplete
    Public Event GameSavingError _
      As EventHandler(Of GameCatalogErrorEventArgs) _
      Implements IGameCatalog.GameSavingError
  End Class
End Namespace